==> EmiR_1.0.0.tar.gz: EmiR/src/Base/Random.cpp <==
/*
###############################################################################
# Emir: EmiR: Evolutionary minimization forR                                  #
# Copyright (C) 2021 Davide Pagano & Lorenzo Sostero                          #
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# any later version.                                                          #
#                                                                             #
# This program is distributed in the hope that it will be useful, but         #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    #
# for more details: <https://www.gnu.org/licenses/>.                          #
###############################################################################
 */


#include "Random.h"

using std::chrono::duration_cast;
using std::chrono::milliseconds;
using std::chrono::system_clock;

Random::Random() {
  m_seed = duration_cast<milliseconds>(system_clock::now().time_since_epoch()).count();
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


Random::Random(uint64_t seed) : m_seed(seed) {}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


double Random::rand() {
  uint64_t x = next();
  return toDouble(x);
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


double Random::rand(double t_min, double t_max) {
  return (t_max - t_min)*rand() + t_min;
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


std::vector<double> Random::randVector(size_t n, double t_min, double t_max) {
  std::vector<double> v(n);
  for (size_t i = 0; i < n; ++i) v[i] = rand(t_min, t_max);
  return v;
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


uint64_t Random::randUInt(uint64_t t_min, uint64_t t_max) {
  uint64_t x, r;
  do {
    x = next();
    r = x % (t_max - t_min);
  } while (x - r > (t_min - t_max));
  return r + t_min;
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

double Random::norm() {
  double u1 = rand();
  double u2 = rand();
  double pi = 3.14159265358979323846;
  return sqrt(-2*log(u1))*cos(2*pi*u2);
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


double Random::norm(double t_mean, double t_sigma) {
  return norm()*t_sigma + t_mean;
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


uint64_t Random::next() {
  uint64_t z = (m_seed += 0x9e3779b97f4a7c15);
  z = (z ^ (z >> 30)) * 0xbf58476d1ce4e5b9;
  z = (z ^ (z >> 27)) * 0x94d049bb133111eb;
  return z ^ (z >> 31);
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


double Random::toDouble(uint64_t x) {
  union U {
    uint64_t i;
    double d;
  };
  U u = { UINT64_C(0x3FF) << 52 | x >> 12 };
  return u.d - 1.0;
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

==> EmiR_1.0.1.tar.gz: EmiR/src/Base/Random.cpp <==
/*
###############################################################################
# Emir: EmiR: Evolutionary minimization forR                                  #
# Copyright (C) 2021 Davide Pagano & Lorenzo Sostero                          #
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# any later version.                                                          #
#                                                                             #
# This program is distributed in the hope that it will be useful, but         #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    #
# for more details: <https://www.gnu.org/licenses/>.                          #
###############################################################################
 */


#include "Random.h"

using std::chrono::duration_cast;
using std::chrono::milliseconds;
using std::chrono::system_clock;

Random::Random() {
  m_seed = duration_cast<milliseconds>(system_clock::now().time_since_epoch()).count();
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


Random::Random(uint64_t seed) : m_seed(seed) {}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


double Random::rand() {
  uint64_t x = next();
  return toDouble(x);
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


double Random::rand(double t_min, double t_max) {
  return (t_max - t_min)*rand() + t_min;
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


std::vector<double> Random::randVector(std::size_t n, double t_min, double t_max) {
  std::vector<double> v(n);
  for (std::size_t i = 0; i < n; ++i) v[i] = rand(t_min, t_max);
  return v;
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


uint64_t Random::randUInt(uint64_t t_min, uint64_t t_max) {
  uint64_t x, r;
  do {
    x = next();
    r = x % (t_max - t_min);
  } while (x - r > (t_min - t_max));
  return r + t_min;
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

double Random::norm() {
  double u1 = rand();
  double u2 = rand();
  double pi = 3.14159265358979323846;
  return sqrt(-2*log(u1))*cos(2*pi*u2);
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


double Random::norm(double t_mean, double t_sigma) {
  return norm()*t_sigma + t_mean;
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


uint64_t Random::next() {
  uint64_t z = (m_seed += 0x9e3779b97f4a7c15);
  z = (z ^ (z >> 30)) * 0xbf58476d1ce4e5b9;
  z = (z ^ (z >> 27)) * 0x94d049bb133111eb;
  return z ^ (z >> 31);
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


double Random::toDouble(uint64_t x) {
  union U {
    uint64_t i;
    double d;
  };
  U u = { UINT64_C(0x3FF) << 52 | x >> 12 };
  return u.d - 1.0;
}
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
